package io.github.apollozhu.lottery.prize

import io.github.apollozhu.lottery.imageAspectFit
import io.github.apollozhu.lottery.utils.AZGridBagConstraints
import io.github.apollozhu.swing.AZJButton
import java.awt.GridBagLayout
import java.awt.Image
import javax.swing.*
import javax.swing.filechooser.FileNameExtensionFilter


data class LotteryPrizeGeneratePanel(val identifier: String) : JPanel() {
    private val nameTextField = JTextField()
    private val countTextField = JTextField()
    private val imagePathButton = AZJButton("選擇圖片")
    private var prizeImage: Image? = null

    init {
        layout = GridBagLayout()
        add(JLabel("名稱"), AZGridBagConstraints(0, 0))
        add(nameTextField, AZGridBagConstraints(1, 0, weightx = 100.0))
        add(JLabel("人數"), AZGridBagConstraints(0, 1))
        add(countTextField, AZGridBagConstraints(1, 1, weightx = 100.0))
        add(JLabel("圖片"), AZGridBagConstraints(0, 2, gridheight = 2, weighty = 100.0))
        add(imagePathButton, AZGridBagConstraints(1, 2, gridheight = 2, weightx = 100.0, weighty = 100.0))
        imagePathButton.addActionListener {
            val chooser = JFileChooser()
            chooser.dialogTitle = "選擇${if (hasPrizeName()) prizeName else "獎品"}圖片"
            chooser.dialogType = JFileChooser.FILES_ONLY
            chooser.fileFilter = FileNameExtensionFilter("Images (jpg/jpeg/gif/png)", "jpg", "jpeg", "gif", "png")
            if (chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
                imagePathButton.text = chooser.selectedFile.path
                prizeImage = ImageIcon(imagePathButton.text).image
                imagePathButton.icon = ImageIcon(imageAspectFit(prizeImage!!, imagePathButton.width - 50, imagePathButton.height - 50))
            }
        }
    }

    val prizeName: String
        get() = nameTextField.text.trim()

    fun hasPrizeName() = prizeName.isNotBlank()

    val prizeCount: String
        get() = countTextField.text.trim()

    val unsafePrizeCount: Int?
        get() = prizeCount.toIntOrNull()

    fun isDataValid() = hasPrizeName()
            && unsafePrizeCount != null
            && unsafePrizeCount!! > 0

    val model: LotteryPrizeModel?
        get() = if (isDataValid()) LotteryPrizeModel(prizeName, unsafePrizeCount!!, prizeImage) else null
}
