package io.github.apollozhu.lottery.utils

import io.github.apollozhu.lottery.settings.LotteryPreferences
import io.github.apollozhu.lottery.utils.RandomSelector.forEachListener
import io.github.apollozhu.util.AZListenable
import java.nio.charset.Charset
import java.nio.file.Files
import java.nio.file.Paths
import java.util.*
import java.util.stream.Collectors
import java.util.stream.Stream
import javax.swing.event.ChangeListener
import javax.swing.event.EventListenerList

object RandomSelector : AZListenable<ChangeListener> {

    private val changeListenerList = EventListenerList()
    override fun getListenerList() = changeListenerList

    init {
        LotteryPreferences.addListener { loadList() }
        loadList()
    }

    var list: MutableList<String> = ArrayList()

    private var currentPath = ""

    fun loadList(stringPath: String = LotteryPreferences.listPath, cs: Charset? = null) {
        val path = Paths.get(stringPath)
        if (stringPath == currentPath || !Files.exists(path)) {
            return
        }
        currentPath = stringPath
        var stream: Stream<String>
        try {
            stream = Files.lines(path, cs ?: Charset.forName("UTF8"))
        } catch (ignored: Exception) {
            stream = Files.lines(path)
        }
        list = stream
                .map { it.replace("\\s+", "") }
                .filter { !it.isBlank() }
                .collect(Collectors.toList())
        forEachListener { it.stateChanged(null) }
    }

    fun add(it: String) = list.add(it)

    fun hasNext(): Boolean = list.size > 0

    fun randomIndex(): Int = (Math.random() * list.size).toInt()

    fun next(): String? = if (hasNext()) list.removeAt(randomIndex()) else null
}
